@@ -75,6 +75,29 @@ setMethod("sds", signature(x="list"),
 	}
 )
 
+setMethod("sds", signature(x="stars"),
+	function(x) {
+		s <- from_stars(x) 
+		if (inherits(s, "SpatRaster")) {
+			sds(s)
+		} else {
+			s
+		}
+	}
+)
+
+setMethod("sds", signature(x="stars_proxy"),
+	function(x) {
+		s <- from_stars(x) 
+		if (inherits(s, "SpatRaster")) {
+			sds(s)
+		} else {
+			s
+		}
+	}
+)
+
+
 setMethod("c", signature(x="SpatRasterDataset"), 
 	function(x, ...) {
 
@@ -105,6 +128,7 @@ setMethod("c", signature(x="SpatRasterDataset"),
 	}
 )
 
+
 setReplaceMethod("[", c("SpatRasterDataset","numeric","missing"),
 	function(x, i, j, value) {
 		if (any(!is.finite(i)) | any(i<1)) {
