@@ -510,10 +510,12 @@ setMethod("mask", signature(x="SpatRaster", mask="SpatVector"),
 
 
 setMethod("project", signature(x="SpatRaster"), 
-	function(x, y, method, mask=FALSE, filename="", ...)  {
+	function(x, y, method, mask=FALSE, align=FALSE, filename="", ...)  {
 	  
 		if (missing(method)) {
 			method <- ifelse(is.factor(x)[1], "near", "bilinear")
+		} else {
+			method <- method[1]
 		}
 		if (method == "ngb") {
 			method <- "near"
@@ -521,15 +523,14 @@ setMethod("project", signature(x="SpatRaster"),
 		}
 		opt <- spatOptions(filename, ...)
 		if (inherits(y, "SpatRaster")) {
-			#x@ptr <- x@ptr$warp(y@ptr, method, opt)
-			x@ptr <- x@ptr$warp(y@ptr, "", method, mask, opt)
+			x@ptr <- x@ptr$warp(y@ptr, "", method, mask[1], align[1], opt)
 		} else {
 			if (!is.character(y)) {
 				warn("project,SpatRaster", "crs should be a character value")
 				y <- as.character(crs(y))
 			}
 			#x@ptr <- x@ptr$warpcrs(y, method, opt)
-			x@ptr <- x@ptr$warp(SpatRaster$new(), y, method, mask, opt)
+			x@ptr <- x@ptr$warp(SpatRaster$new(), y, method, mask, FALSE, opt)
 		}
 		messages(x, "project")
 	}
@@ -601,7 +602,7 @@ setMethod("resample", signature(x="SpatRaster", y="SpatRaster"),
 			warn("project", "argument 'method=ngb' is deprecated, it should be 'method=near'")
 		}
 		opt <- spatOptions(filename, ...)
-		x@ptr <- x@ptr$warp(y@ptr, "", method, FALSE, opt)
+		x@ptr <- x@ptr$warp(y@ptr, "", method, FALSE, FALSE, opt)
 		messages(x, "resample")
 	}
 )
