@@ -1,6 +1,10 @@
+#ifndef SPATFACTOR_GUARD
+#define SPATFACTOR_GUARD
+
+
 #include <vector>
 #include <string>
-
+#include <numeric>
 //#include "NA.h"
 #include <algorithm>
 
@@ -131,3 +135,5 @@ public:
 	SpatFactor(){} ;
 };
 
+#endif
+
